#!/usr/bin/python

import ctypes

# represents struct Color from C turtle graphics library
class Color(ctypes.Structure):
    _fields_ = [
        ("r",ctypes.c_uint8),
        ("g",ctypes.c_uint8),
        ("b",ctypes.c_uint8)
    ]
    def __init__(self,r,g,b):
        self.r = r
        self.g = g
        self.b = b

class CTurtleException(Exception): pass

# encapsulates the C turtle graphics library
class CTurtle(object):
    c = ctypes.cdll.LoadLibrary("libCTurtle.so")
    c.get_last_error.restype = ctypes.c_char_p # set the return type to string explicitly (as opposed to numeric pointer)
    
    #def __init__(self):
    #    pass 
    
    @classmethod
    def new(self,w,h):
        r = CTurtle()
        r.img = self.c.create_image(w,h)
        return r
    
    def __del__(self):
        self.c.destroy_image(self.img)
    
    def set_xy(self,x,y):
        if not self.c.turtle_set_xy(self.img, x, y):
            raise CTurtleException("Tried to setxy out of bounds")
    
    def move(self, x, y):
        if not self.c.turtle_move(self.img, x, y):
            raise CTurtleException("Tried to move out of bounds")
        
    def forward(self, d):
        if not self.c.turtle_forward(self.img, d):
            raise CTurtleException("Tried to go forward out of bounds")
    
    # TODO - add the binding for face_north()!
    # TODO - add the binding for left()!
    # TODO - add the binding for right()!
    # Hint: these bindings look just like all the others...

    def pen_up(self):
        self.c.turtle_pen_up(self.img)

    def pen_down(self):
        self.c.turtle_pen_down(self.img);        

    def set_color(self, color):
        self.c.turtle_set_color(self.img, color)

    def do_pixel(self, x, y, color):
        self.c.do_pixel(self.img, x, y, color)
    
    def do_line(self, x0, y0, x1, y1, color):
        self.c.do_line(self.img, x0, y0, x1, y1, color)
    
    def do_clear(self, color):
        self.c.do_clear(self.img, color)

    def do_invert(self):
        self.c.do_invert(self.img)

    def do_reverse_rgb(self):
        self.c.do_reverse_rgb(self.img)

    def write_tga(self, filename):
        return self.c.write_tga(filename, self.img)

    def write_ppm(self, filename, ascii=False):
        return self.c.write_ppm(filename, self.img, ascii)

    @classmethod
    def read_tga(self, filename):
        r = CTurtle()
        r.img = self.c.read_tga(filename)
        if not r.img:
            raise CTurtleException(self.c.get_last_error())
        return r
        
    @classmethod
    def read_ppm(self, filename):
        r = CTurtle()
        r.img = self.c.read_ppm(filename)
        if not r.img:
            raise CTurtleException(self.c.get_last_error())
        return r
        
    @classmethod
    def get_last_error():
        return self.c.get_last_error()
