#!/usr/bin/python

import ctypes
import sys,os,re

from CTurtle import *

def hilbert_l(img, lvl, w):
    if lvl==0: return
    img.right(90)
    hilbert_r(img, lvl-1, w)
    img.forward(w)
    img.left(90)
    hilbert_l(img, lvl-1, w)
    img.forward(w)
    hilbert_l(img, lvl-1, w)
    img.left(90)
    img.forward(w)
    hilbert_r(img, lvl-1, w)
    img.right(90)

def hilbert_r(img, lvl, w):
    if lvl==0: return
    img.left(90)
    hilbert_l(img, lvl-1, w)
    img.forward(w)
    img.right(90)
    hilbert_r(img, lvl-1, w)
    img.forward(w)
    hilbert_r(img, lvl-1, w)
    img.right(90)
    img.forward(w)
    hilbert_l(img, lvl-1, w)
    img.left(90)

def hilbert(img, lvl, w):
    hilbert_l(img, lvl, w)

def usage(msg=None):
    print "CTurtle test suite for Python.\nSyntax:\n  ./pyturtle_test.py <test#> <outfile> [infile]\n"
    if msg: print msg
    sys.exit(1)

def verbose_write_ppm(filename, img, ascii):
    img.write_ppm(filename, ascii)
    #print "Success wrote '%s'." % filename
    # no need to print error explcitly -- the python exception handler will do that

def verbose_write_tga(filename, img):
    img.write_tga(filename)
    #print "Success wrote '%s'." % filename
    # no need to print error explcitly -- the python exception handler will do that

def write_test_image_outputs(img, outfile_base):
    verbose_write_ppm("test-out/%s-a.ppm" % outfile_base , img, True)
    verbose_write_ppm("test-out/%s-b.ppm" % outfile_base , img, False)
    verbose_write_tga("test-out/%s.tga"   % outfile_base , img)


# used for multiple tests
def make_simple_base_image():
    (w,h) = (3,5)
    img = CTurtle.new(w,h)
    
    # r g b dots going down the middle
    img.do_pixel(1,0,Color(255,0,0))
    img.do_pixel(1,2,Color(0,255,0))
    img.do_pixel(1,4,Color(0,0,255))
    
    # 4 corners in white
    img.do_pixel(0  ,0  ,Color(255,255,255))
    img.do_pixel(0  ,h-1,Color(255,255,255))
    img.do_pixel(w-1,0  ,Color(255,255,255))
    img.do_pixel(w-1,h-1,Color(255,255,255))
    
    return img

def read_image(filename):
    try:
        if   filename.endswith(".tga"): img = CTurtle.read_tga(filename)
        elif filename.endswith(".ppm"): img = CTurtle.read_ppm(filename)
        else:
            print "Error: %s doesn't have an appropriate extension." % filename
            sys.exit(1)
        return img
    except CTurtleException,ex:
        print "%s: %s" % (filename,ex)
        sys.exit(1)

# write the given image to the given filename in the given format
def write_output(img, out_filename):
    format = get_format(out_filename)
    if   format=='TARGA':      verbose_write_tga(out_filename, img)
    elif format=='ASCII_PPM':  verbose_write_ppm(out_filename, img, True)
    elif format=='BINARY_PPM': verbose_write_ppm(out_filename, img, False)
    else:
        print("Tried to write an invalid format!")
        sys.exit(1)

def get_format(filename):
    if   filename.endswith(".tga"):   return 'TARGA'
    elif filename.endswith(".a.ppm"): return 'ASCII_PPM'
    elif filename.endswith(".ppm"):   return 'BINARY_PPM'
    else: return None

def do_test(test_id, out_filename, in_filename=None):
    print "Running test %d (%s)..." % (test_id, out_filename)
    
    if test_id==1:
        (w,h) = (3,5)
        img = make_simple_base_image()
        
        write_output(img, out_filename)

    elif test_id==2:
        # same as test 1, but invert at the end
        img = make_simple_base_image()
        
        img.do_invert()

        write_output(img, out_filename)
        
    elif test_id==3:
        # same as test 1, but reverse rgb at the end
        img = make_simple_base_image()
        
        img.do_reverse_rgb()

        write_output(img, out_filename)
        
    elif test_id==4:
        # a colored triforce pattern on the left of a 150x60 dark blue background
        (w,h) = (150,60)
        img = CTurtle.new(w,h)
        
        # this line will be covered up by the clear operation and shouldn't be seen
        img.do_line( 0,0, w-1, h-1, Color(255,255,255)) # white
        
        # clear the canvas
        img.do_clear(Color(0,0,40)) # dark blue background
        
        # upper triangle
        img.do_line( 50,20, 60,30, Color(255,000,000)) # red
        img.do_line( 50,20, 40,30, Color(000,255,000)) # green
        img.do_line( 40,30, 60,30, Color(000,000,255)) # blue

        # lower left triangle
        img.do_line( 50,40, 60,30, Color(255,000,255)) # magenta
        img.do_line( 50,40, 70,40, Color(000,255,255)) # cyan
        img.do_line( 70,40, 60,30, Color(255,255,000)) # yellow

        # lower right triangle
        img.do_line( 50,40, 40,30, Color(128,000,000)) # dk red
        img.do_line( 50,40, 30,40, Color(000,128,000)) # dk green
        img.do_line( 30,40, 40,30, Color(000,000,128)) # dk blue

        # vertical lines on the sides
        img.do_line( 20,10, 20,50, Color(128,128,128)) # grey
        img.do_line( 80,10, 80,50, Color(128,128,128)) # grey
        
        write_output(img, out_filename)
        
    elif test_id==5:
        # read the given file (auto-detecting type by extension) and output it (no changes)
        if not in_filename: usage("Need input filename for this test")
        img = read_image(in_filename)
        
        # no need to check for error; python exception will occur if there's a problem
        
        write_output(img, out_filename)
        
    elif test_id==6:
        # read the given file (auto-detecting type by extension) and output it with rgb reversed
        if not in_filename: usage("Need input filename for this test")
        img = read_image(in_filename)
        
        # no need to check for error; python exception will occur if there's a problem
        
        img.do_reverse_rgb()
        
        write_output(img, out_filename)
        
    elif test_id==7:
        # read the given file (auto-detecting type by extension) and invert it, then draw most of test 2 on top of it
        if not in_filename: usage("Need input filename for this test")
        img = read_image(in_filename)
        
        img.do_invert()
        
        # upper triangle
        img.do_line( 50,20, 60,30, Color(255,000,000)) # red
        img.do_line( 50,20, 40,30, Color(000,255,000)) # green
        img.do_line( 40,30, 60,30, Color(000,000,255)) # blue

        # lower left triangle
        img.do_line( 50,40, 60,30, Color(255,000,255)) # magenta
        img.do_line( 50,40, 70,40, Color(000,255,255)) # cyan
        img.do_line( 70,40, 60,30, Color(255,255,000)) # yellow

        # lower right triangle
        img.do_line( 50,40, 40,30, Color(128,000,000)) # dk red
        img.do_line( 50,40, 30,40, Color(000,128,000)) # dk green
        img.do_line( 30,40, 40,30, Color(000,000,128)) # dk blue

        # vertical lines on the sides
        img.do_line( 20,10, 20,50, Color(128,128,128)) # grey
        img.do_line( 80,10, 80,50, Color(128,128,128)) # grey
        
        write_output(img, out_filename)
        
        
    elif test_id==8:
        # the "hello world" of turtle graphics -- drawing one line with the default settings in the default direction
        (w,h) = (50,50)
        img = CTurtle.new(w,h)
        
        img.forward(20)
        
        write_output(img, out_filename)
        
    elif test_id==9:
        # same as 8, but turn a bit first
        (w,h) = (50,50)
        img = CTurtle.new(w,h)
        
        img.right(30)
        img.forward(20)
        
        write_output(img, out_filename)
        
    elif test_id==10:
        # simple triangle with different colors on each edge (red/orange/yellow)
        (w,h) = (50,50)
        img = CTurtle.new(w,h)
        
        img.set_xy(5,5)
        
        for i in xrange(3):
            img.set_color(Color(255,i*120,0))
            img.right(120)
            img.forward(30)
        
        write_output(img, out_filename)
        
    elif test_id==11:
        # draw a magenta box with a cyan X on it a bit off center
        (w,h) = (60,45)
        img = CTurtle.new(w,h)
        
        img.set_xy(w/2+3, h/2-3)
        
        img.set_color(Color(255,0,255))
        img.set_xy(10,10)
        img.move(40,10)
        img.move(40,40)
        img.move(10,40)
        img.move(10,10)
        
        img.set_color(Color(0,255,255))
        img.move(40,40)
        img.set_xy(40,10)
        img.move(10,40)
        
        write_output(img, out_filename)
        
    elif test_id==12:
        # draw a "zipper" type shape going from blue to cyan
        (w,h) = (50,80)
        img = CTurtle.new(w,h)
        
        img.set_xy(w/2,h-5)
        for i in xrange(4):
            img.set_color(Color(0,85*i,255))
            
            img.face_north()
            img.forward(10)
            
            img.left(100)
            img.forward(10)
            
            img.face_north()
            img.forward(10)
            
            img.right(100)
            img.forward(10)
            
        write_output(img, out_filename)
        
    elif test_id==13:
        # draw a dashed hexagon
        (w,h) = (170,200)
        img = CTurtle.new(w,h)
        
        n=6
        img.set_xy(3,140)
        for i in xrange(n):
            
            for j in xrange(3):
                img.pen_up()
                img.forward(10)
                img.pen_down()
                img.forward(20)
            img.right(360/n)
            
        write_output(img, out_filename)
        
    elif test_id==14:
        # draw a totally sweet hilbert curve fractal
        (w,h) = (256,256)
        img = CTurtle.new(w,h)
        
        img.do_clear(Color(15,0,30))
        
        img.set_color(Color(255,128,0))
        
        img.set_xy(1,h-2)
        hilbert(img,6,4)
        
        write_output(img, out_filename)

    elif test_id==15:
        # make two images at once
        if "%d" not in out_filename:
            print("This test must have an output filename containing '%d', as it outputs multiple numbered files at once.\n")
            sys.exit(1)
        w=70; h=60;
        img1 = CTurtle.new(w,h)
        img2 = CTurtle.new(w+5,h+9)
        
        img1.do_clear(Color(15,0,30))
        img2.do_clear(Color(45,2,4))
        
        img1.set_color(Color(255,128,0))
        img2.set_color(Color(0,64,255))
        
        img1.set_xy(w/2-9,  h/2-5)
        img2.set_xy(w/2-12, h/2-17)
        
        img1.forward( 13)
        img2.forward(-33)
        
        img1.right(30)
        img2.left (45)
        
        img1.forward( 11)
        img2.forward(-19)
        
        out_filename1 = out_filename % (1)
        out_filename2 = out_filename % (2)
        
        write_output(img1, out_filename1)
        write_output(img2, out_filename2)
        
    else:
        usage("Invalid test id")
        
if len(sys.argv)<3:
    usage()
test_id = int(sys.argv[1])
out_filename = sys.argv[2]
in_filename = None
if len(sys.argv)>=4:
    in_filename = sys.argv[3]

do_test(test_id, out_filename, in_filename)
print "OK."